<?php
/**
 * The complete management for the Zoho-CF7 plugin through out the site.
 *
 * @since      1.0.0
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/includes
 * @author     MakeWebBetter <https://makewebbetter.com>
 */

/**
 * The complete management for the ajax handlers.
 *
 * @since      1.0.0
 * @package    Mwb_Cf7_Integration_With_Zoho
 * @subpackage Mwb_Cf7_Integration_With_Zoho/includes
 * @author     MakeWebBetter <https://makewebbetter.com>
 */
class Mwb_Cf7_Integration_With_Zoho_Ajax_Handler {

	/**
	 * Get default response.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function get_default_response() {
		return array(
			'status'  => false,
			'message' => esc_html__( 'Something went wrong!!', 'mwb-cf7-integration-with-zoho' ),
		);
	}

	/**
	 * Ajax handler :: Handles all ajax callbacks.
	 *
	 * @since 1.0.0
	 */
	public function mwb_zcf7_ajax_callback() {

		/* Nonce verification */
		check_ajax_referer( 'mwb_zcf7_nonce', 'nonce' );

		$event    = ! empty( $_POST['event'] ) ? sanitize_text_field( wp_unslash( $_POST['event'] ) ) : '';
		$response = $this->get_default_response();

		if ( ! empty( $event ) ) {
			$data = $this->$event( $_POST );
			if ( $data ) { // @codingStandardsIgnoreLine
				$response['status']  = true;
				$response['message'] = esc_html__( 'Success', 'mwb-cf7-integration-with-zoho' );

				$response = $this->maybe_add_data( $response, $data );
			}
		}

		wp_send_json( $response );

	}

	/**
	 * Merge additional data to response.
	 *
	 * @param array $response An array of response.
	 * @param array $data     An array of data to merge in response.
	 * @return array
	 */
	public function maybe_add_data( $response, $data ) {

		if ( is_array( $data ) ) {
			$response['data'] = $data;
		}

		return $response;
	}

	/**
	 * Referesh access tokens.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function refresh_access_token() {

		$response        = array( 'success' => false );
		$zoho_api        = Zcf7_Api::get_instance();
		$response['msg'] = esc_html__( 'Something went wrong! Check your credentials and authorize again', 'mwb-cf7-integration-with-zoho' );

		if ( $zoho_api->renew_access_token() ) {
			$token_expiry  = Mwb_Cf7_Integration_With_Zoho_Helper::get_token_expiry_details();
			$token_message = sprintf( 'Access token will expire in %s minute(s).', $token_expiry );
			$response      = array(
				'success'       => true,
				'msg'           => esc_html__( 'Success', 'mwb-cf7-integration-with-zoho' ),
				'token_message' => $token_message,
			);
		}
		return $response;
	}

	/**
	 * Reauthorize account
	 *
	 * @since 1.0.0
	 * @return string.
	 */
	public function reauthorize_zcf7_account() {

		$response = array( 'success' => false );
		$zoho_api = Zcf7_Api::get_instance();
		$auth_url = $zoho_api->get_auth_code_url();

		if ( ! $auth_url ) {
			$response['msg'] = esc_html__( 'Something went wrong! Check your credentials and authorize again', 'mwb-cf7-integration-with-zoho' );
			delete_option( 'mwb_is_crm_active', false );
			return $response;
		}

		$response = array(
			'success' => true,
			'url'     => $auth_url,
		);

		return $response;
	}

	/**
	 * Revoke account access.
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function revoke_zcf7_access() {

		delete_option( 'mwb_is_crm_active' );
		delete_option( 'mwb_zcf7_zoho_token_data' );

		return true;
	}

	/**
	 * Fetch zoho objects.
	 *
	 * @since 1.0.0
	 * @return mixed
	 */
	public function fetch_zoho_modules() {
		$zoho_api = Zcf7_Api::get_instance();
		return $zoho_api->get_modules_data();
	}

	/**
	 * Fetch object fields.
	 *
	 * @param array $data An array of ajax data.
	 * @since 1.0.0
	 * @return array
	 */
	public function fetch_module_fields( $data = array() ) {

		// Get zoho crm module fields.
		$module_data = array();
		$module      = ! empty( $data['object'] ) ? sanitize_text_field( wp_unslash( $data['object'] ) ) : '';
		$force       = ! empty( $data['force'] ) ? sanitize_text_field( wp_unslash( $data['force'] ) ) : false;
		$zoho_api    = Zcf7_Api::get_instance();

		$module_data['crm_fields'] = $zoho_api->get_module_fields( $module, $force );

		return $module_data;
	}

	/**
	 * Fetch form fields.
	 *
	 * @param array $data An array of ajax posted data.
	 * @since 1.0.0
	 * @return array
	 */
	public function fetch_form_fields( $data = array() ) {

		$form_data = array();
		$form_id   = ! empty( $data['form_id'] ) ? sanitize_text_field( wp_unslash( $data['form_id'] ) ) : '';
		$helper    = new Mwb_Cf7_Integration_With_Zoho_Helper();
		$response  = $helper->parse_form_fields( $form_id );

		if ( ! empty( $response ) && is_array( $response ) ) {
			foreach ( $response as $key => $value ) {
				$form_data[ $key ] = $value['label'];

			}
		}

		$form_data = array(
			'Contact Form Fields' => $form_data,
		);

		$settings['field_type'] = array(
			'label'   => esc_html__( 'Field Type', 'mwb-cf7-integration-with-zoho' ),
			'options' => array(
				'standard_field' => esc_html__( 'Standard Fields', 'mwb-cf7-integration-with-zoho' ),
				'custom_value'   => esc_html__( 'Custom Value', 'mwb-cf7-integration-with-zoho' ),
			),
		);

		$settings['field_value'] = array(
			'label'   => esc_html__( 'Field Value', 'mwb-cf7-integration-with-zoho' ),
			'options' => $form_data,
		);

		$settings['custom_value'] = array(
			'label' => esc_html__( 'Custom Value', 'mwb-cf7-integration-with-zoho' ),
		);

		return $settings;
	}

	/**
	 * Toggle feed status.
	 *
	 * @param array $data An array of ajax posted data.
	 * @since 1.0.0
	 * @return bool
	 */
	public function toggle_feed_status( $data = array() ) {

		$feed_id  = ! empty( $data['feed_id'] ) ? sanitize_text_field( wp_unslash( $data['feed_id'] ) ) : '';
		$status   = ! empty( $data['status'] ) ? sanitize_text_field( wp_unslash( $data['status'] ) ) : '';
		$response = Mwb_Cf7_Integration_With_Zoho_Helper::change_post_status( $feed_id, $status );
		return $response;
	}

	/**
	 * Trash feeds.
	 *
	 * @param array $data An array of ajax posted data.
	 * @since 1.0.0
	 */
	public function trash_feeds_from_list( $data = array() ) {

		$feed_id = ! empty( $data['feed_id'] ) ? sanitize_text_field( wp_unslash( $data['feed_id'] ) ) : '';
		$trash   = wp_trash_post( $feed_id );

		if ( $trash ) {
			return true;
		}
		return false;
	}

	/**
	 * Clear sync log.
	 *
	 * @param array $data An array of ajax posted data.
	 * @since 1.0.0
	 * @return array Response array.
	 */
	public function clear_sync_log( $data = array() ) {
		Mwb_Cf7_Integration_With_Zoho_Admin::delete_sync_log();
		return array( 'success' => true );
	}

	/**
	 * Download logs.
	 *
	 * @param array $data An arraay of ajax posted data.
	 * @since 1.0.0
	 * @return array Response array.
	 */
	public function download_sync_log( $data = array() ) {

		global $wpdb;
		$table_name     = $wpdb->prefix . 'mwb_zcf7_log';
		$log_data_query = "SELECT * FROM {$table_name} ORDER BY `id` DESC";
		$log_data       = $wpdb->get_results( $log_data_query, ARRAY_A ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$path           = Mwb_Cf7_Integration_With_Zoho_Helper::create_log_folder( 'zoho-cf7-logs' );
		$log_dir        = $path . '/mwb-zcf7-sync-log.log';

		foreach ( $log_data as $key => $value ) {

			$value['zoho_id'] = ! empty( $value['zoho_id'] ) ? $value['zoho_id'] : '-';

			$log  = 'FEED ID: ' . $value['feed_id'] . PHP_EOL;
			$log .= 'FEED : ' . $value['feed'] . PHP_EOL;
			$log .= 'ZOHO ID : ' . $value['zoho_id'] . PHP_EOL;
			$log .= 'ZOHO OBJECT : ' . $value['zoho_object'] . PHP_EOL;
			$log .= 'TIME : ' . gmdate( 'd-m-Y h:i A', esc_html( $value['time'] ) ) . PHP_EOL;
			$log .= 'REQUEST : ' . wp_json_encode( maybe_unserialize( $value['request'] ) ) . PHP_EOL;
			$log .= 'RESPONSE : ' . wp_json_encode( maybe_unserialize( $value['response'] ) ) . PHP_EOL;
			$log .= '------------------------------------' . PHP_EOL;
			file_put_contents( $log_dir, $log, FILE_APPEND ); // @codingStandardsIgnoreLine
		}

		return array(
			'success'  => true,
			'redirect' => admin_url( '?mwb_download=1' ),
		);
	}

	/**
	 * Enable datatable.
	 *
	 * @param mixed $data An array of ajax posted data.
	 */
	public function get_datatable_data_cb( $data = array() ) {

		$request = $_GET; // @codingStandardsIgnoreLine
		$offset  = $request['start'];
		$limit   = $request['length'];

		global $wpdb;
		$table_name     = $wpdb->prefix . 'mwb_zcf7_log';
		$log_data_query = $wpdb->prepare( "SELECT * FROM {$table_name} ORDER BY `id` DESC LIMIT %d OFFSET %d ", $limit, $offset ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$log_data       = $wpdb->get_results( $log_data_query, ARRAY_A ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$count_query    = "SELECT COUNT(*) as `total_count` FROM {$table_name}";
		$count_data     = $wpdb->get_col( $count_query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$total_count    = $count_data[0];
		$data           = array();

		foreach ( $log_data as $key => $value ) {

			$data_href = Mwb_Cf7_Integration_With_Zoho_Helper::get_zoho_link( $value );
			if ( in_array( $value['zoho_object'], array( 'Note', 'Task' ) ) ) { // @codingStandardsIgnoreLine
				$link = $value['zoho_id'];
			} elseif ( ! empty( $data_href ) ) {
				$link = '<a href="' . $data_href . '" target="_blank">' . $value['zoho_id'] . '</a>';
			} else {
				$link = $value['zoho_id'];
			}

			$value['zoho_id'] = ! empty( $value['zoho_id'] ) ? $value['zoho_id'] : '-';

			$temp   = array(
				'<span class="dashicons dashicons-plus-alt"></span>',
				$value['feed'],
				$value['feed_id'],
				$value['zoho_object'],
				$link,
				$value['event'],
				gmdate( 'd-m-Y h:i A', esc_html( $value['time'] ) ),
				wp_json_encode( maybe_unserialize( $value['request'] ) ),
				wp_json_encode( maybe_unserialize( $value['response'] ) ),
			);
			$data[] = $temp;
		}

		$json_data = array(
			'draw'            => intval( $request['draw'] ),
			'recordsTotal'    => $total_count,
			'recordsFiltered' => $total_count,
			'data'            => $data,
		);

		wp_send_json( $json_data );
	}

}
